#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains all constant definitions used by Plot Twist
"""

from __future__ import print_function, division, absolute_import

__author__ = "Tomas Poveda"
__license__ = "MIT"
__maintainer__ = "Tomas Poveda"
__email__ = "tpovedatd@gmail.com"

from artellapipe.core import defines


# Defines the asset type used for Props
PLOTTWIST_PROP_ASSETS = 'Props'

# Defines the asset type used for Characters
PLOTTWIST_CHARACTERS_ASSETS = 'Characters'

# Defines of textures asset type for Plot Twist
PLOTTWIST_TEXTURES_ASSET_TYPE = defines.ARTELLA_TEXTURES_ASSET_TYPE

# Defines of model asset type for Plot Twist
PLOTTWIST_MODEL_ASSET_TYPE = defines.ARTELLA_MODEL_ASSET_TYPE

# Defines of shading asset type for Plot Twist
PLOTTWIST_SHADING_ASSET_TYPE = defines.ARTELLA_SHADING_ASSET_TYPE

# Defines of rig asset type for Plot Twist
PLOTTWIST_RIG_ASSET_TYPE = defines.ARTELLA_RIG_ASSET_TYPE

# Defines of groom asset type for Plot Twist
PLOTTWIST_GROOM_ASSET_TYPE = defines.ARTELLA_GROOM_ASSET_TYPE

# Defines Alembic extension used by Plot Twist
PLOTTWIST_ALEMBIC_EXTENSION = '.abc'

# Defines Alembic extension used by Plot Twist
PLOTTWIST_MODEL_EXTENSION = '.ma'

# Defines Texture extensions used by Plot Twist
PLOTTWIST_TEXTURE_EXTENSIONS = ['.tx', '.png', '.jpg', '.jpeg']

# Defines Alembic extension used by Plot Twist
PLOTTWIST_SHADING_EXTENSION = '.ma'

# Defines Alembic extension used by Plot Twist
PLOTTWIST_RIG_EXTENSION = '.ma'
